/* 
 * FILE --- fuzzy_sys_ex1.cc
 *
 * PURPOSE --- To provide an example use of the class for 4'th
 * order Runge-Kutta simulation, RK4_SIM, and the class for implementing
 * n-input 1-output Takagi-Sugeno (TS) fuzzy system, FUZZY_TS_SYS.
 *
 * Implements the main function of a simple illustrative example of the usage of the
 * Takagi-Sugeno fuzzy system.
 *
 */

// Include Files
#include <stdlib.h>
#include <stdio.h>
#include "fuzzy_ts_sys.hh"


void main(void) 
{
  // Simple example of use of the fuzzy system 
  double e[2], u;
  int in_mf[2] = {5, 5};
  double output_gain = 10.0;
  //  double input_gains[2] = {2.0/PI, 4.0/PI};
  double input_gains[2] = {4.0/PI, 2.0/PI};
  double **ts_coeff;

  ts_coeff = new double*[25];
  for(int ii = 0; ii < 25; ii++)
    ts_coeff[ii] = new double[1];

  ts_coeff[0][0] = -1.0;
  ts_coeff[1][0] = -1.0;
  ts_coeff[2][0] = -1.0;
  ts_coeff[3][0] = -0.5;
  ts_coeff[4][0] = 0.0;

  ts_coeff[5][0] = -1.0;
  ts_coeff[6][0] = -1.0;
  ts_coeff[7][0] = -0.5;
  ts_coeff[8][0] = 0.0;
  ts_coeff[9][0] = 0.5;

  ts_coeff[10][0] = -1.0;
  ts_coeff[11][0] = -0.5;
  ts_coeff[12][0] = 0.0;
  ts_coeff[13][0] = 0.5;
  ts_coeff[14][0] = 1.0;

  ts_coeff[15][0] = -0.5;
  ts_coeff[16][0] = 0.0;
  ts_coeff[17][0] = 0.5;
  ts_coeff[18][0] = 1.0;
  ts_coeff[19][0] = 1.0;

  ts_coeff[20][0] = 0.0;
  ts_coeff[21][0] = 0.5;
  ts_coeff[22][0] = 1.0;
  ts_coeff[23][0] = 1.0;
  ts_coeff[24][0] = 1.0;


  FUZZY_TS_SYS my_fuzzy_ts(2, 0, in_mf);

  my_fuzzy_ts.set_input_gains(input_gains);
  my_fuzzy_ts.set_ts_coeff((const double **)ts_coeff);

  // Crips inputs.
  //  e[0] = 0.0;                 
  //  e[1] = (PI/8.0 - PI/32.0);
  e[1] = 0.0;                 
  e[0] = (PI/8.0 - PI/32.0);

  u = output_gain*my_fuzzy_ts.control(e,e);
  
  printf("e = %f, edot = %f, u = %f\n",e[0],e[1],u); 

  for(int ii = 0; ii < 25; ii++)
    delete [] ts_coeff[ii];
  delete [] ts_coeff;
}



