/*
 * FILE --- random.cc
 */

// Include Files
#include "random.hh"


void Randomize(void)
{
  /* Randomize() initializes the pseudo-random number generator 
     by seeding the generator with some function of time */

  int stime;
  long int ltime;
  
  ltime = time(NULL);
  stime = (unsigned) ltime/2;
  srand(stime);
}


double Rand_Num(void)
{
  /* Rand_Num() returns a pseudo-random number between 0.0 and 1.0 
     by calling the function rand() from <stdlib.h>
     If the generated numbers seeme unusual (e.g. all 0's), you may 
     need to modify the MaxRandom value according to the rand() 
     function on your computer system.
     */
  
  int MaxRandom = 2147483647; /* = 2^31 - 1 */
  /* int MaxRandom = 32767;*/  /*  = 2^15 -1 */ 
  
  double number;

  number = (double)rand() / (double)MaxRandom;
  //  number = (double)rand() / (double)RAND_MAX;
  return (number);
}


int Flip(double chance)
{
  /* This function returns TRUE (1) with probability (chance)
     and returns FALSE (0) with probability (1-chance).
     0.0 <= chance <= 1.0 */

  return ( (chance > Rand_Num()) ? 1 : 0);
}


int Rand_Int(int high, int low)
{
  /* Rand_Int returns an integer between high and low */
  /* with equal probability for all numbers (including high & low) */

  int num;
  
  // Make sure that low <= num <= high
  if (low>high)
    {
      num = (int)(Rand_Num()*(low-high+1)) + high;
    }
  else
    {
      num = (int)(Rand_Num()*(high-low+1)) + low;
    }

  return(num);
}

double Rand_Double(double high, double low)
{
  /* Rand_Double returns a double  between high and low */
  /* with equal probability for all numbers */

  double num;

  // Make sure that low <= num <= high
  if (low>high)
    {
      num = (double)(Rand_Num()*(low-high)) + high;
    }
  else
    {
      num = (double)(Rand_Num()*(high-low)) + low;
    }

  return(num);
}

double Power(int x, int y)
{
  /* result = x^y 
     This only works for integer values, and is faster than the function
     pow() in math.h 
     */


  int ii; // Index variables for the loops
  double result;

  result = 1.0;

  if (x == 0)
    {
      result = 0.0;
    }
  else
    {
      if ( y < 0)
	{
	  for (ii = 0; ii > y; ii--)
	    result = result / x ;
	}
      
      if ( y > 0)
	{
	  for (ii = 0; ii < y ; ii++)
	    result = result * x ;
	}
    }

  return(result);
}
